/*
** Module   : WindowWidget.java
** Abstract : server side window widget implementation
**
** Copyright (c) 2005-2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ----------------Description-----------------
** 001 NVS 20051026   @23498 Created initial version.
** 002 GES 20060803   @28385 Major cleanup and refactoring.
** 003 SIY 20061201   @31528 Ignore attempts to change window visibility
**                           for CHUI.
** 004 GES 20080320   @37595 Added TYPE attribute.
** 005 SVL 20081202   @40751 setVisible(false) should have the same effect
**                           as LogicalTerminal.hideAll(true).
** 006 SIY 20081224   @40990 Replaced hideAll() in setVisible() with call 
**                           to dedicated client API.
** 007 SIY 20101004          Added necessary method stubs.
** 008 SIY 20101005          Icon-related method stubs.
** 009 SIY 20101007          Added asWidget() method.
** 010 SIY 20111101          Updated imports.
** 011 CA  20130112          Extracted into CommonWindow interface all window
**                           APIs.
** 012 CA  20130221          Refactored usage of TYPE attribute.
** 013 CA  20130223          Removed assign, as it window assignment is done using LogicalTerminal
**                           APIs.
*/ 

package com.goldencode.p2j.ui;

import com.goldencode.p2j.util.*;
 
/**
 * This class represents the only window widget that exists in a CHUI
 * environment.
 * <p>
 * The window widget differs from all the other widgets also in this: it is 
 * the only widget that is not contained in a frame, but rather contains
 * frames (and other top-level components) itself. 
 */
public class WindowWidget
extends BaseEntity
implements CommonWindow
{
   /** Reference to our configuration which is pre-cast to the proper type. */
   private WindowConfig wcfg = null;
   
   /**
    * Default constructor.  It is package private because an instance must
    * only be created in a controllable way.
    */
   WindowWidget()
   {
      super(new WindowConfig(ComponentConfig.WINDOW, null));
      wcfg = (WindowConfig) config;
      
      setResourceType("WINDOW");
   }

   /**
    * Reports if this widget object is valid for use.
    *
    * @return   Always <code>true</code>.
    */
   public boolean isValid()
   {
      return true;
   }

   /**
    * Gets the KEEP-FRAME-Z-ORDER writable attribute.
    *
    * @return   The current value of the KEEP-FRAME-Z-ORDER attribute.
    */
   public logical isKeepFrameZOrder()
   {
      return new logical(wcfg.isKeepFrameZOrder());
   }

   /**
    * Sets the KEEP-FRAME-Z-ORDER writable attribute.
    *
    * @param    keep
    *           The new value for the KEEP-FRAME-Z-ORDER attribute.
    */
   public void setKeepFrameZOrder(logical keep)
   {
      setKeepFrameZOrder(keep.booleanValue()); 
   }

   /**
    * Sets the KEEP-FRAME-Z-ORDER writable attribute.
    *
    * @param    keep
    *           The new value for the KEEP-FRAME-Z-ORDER attribute.
    */
   public void setKeepFrameZOrder(boolean keep)
   {
      wcfg.setKeepFrameZOrder(keep);
      LogicalTerminal.pushWindow(wcfg);
   }

   /**
    * Gets the MAX-HEIGHT-CHARS writable attribute.
    *
    * @return current value of MAX-HEIGHT-CHARS attribute
    */
   public decimal getMaxHeightChars()
   {
      return new decimal(wcfg.getMaxHeightChars());
   }

   /**
    * Sets the MAX-HEIGHT-CHARS writable attribute.
    *
    * @param    max
    *           The new value for the MAX-HEIGHT-CHARS attribute.
    */
   public void setMaxHeightChars(decimal max)
   {
      setMaxHeightChars(max.doubleValue());
   }

   /**
    * Sets the MAX-HEIGHT-CHARS writable attribute.
    *
    * @param    max
    *           The new value for the MAX-HEIGHT-CHARS attribute.
    */
   public void setMaxHeightChars(double max)
   {
      wcfg.setMaxHeightChars(max);
      LogicalTerminal.pushWindow(wcfg);
   }

   /**
    * Gets the MAX-WIDTH-CHARS writable attribute.
    *
    * @return   The current value of MAX-WIDTH-CHARS attribute.
    */
   public decimal getMaxWidthChars()
   {
      return new decimal(wcfg.getMaxWidthChars());
   }

   /**
    * Sets the MAX-WIDTH-CHARS writable attribute.
    *
    * @param    max
    *           The new value for the MAX-WIDTH-CHARS attribute.
    */
   public void setMaxWidthChars(decimal max)
   {
      setMaxWidthChars(max.doubleValue());
   }

   /**
    * Sets the MAX-WIDTH-CHARS writable attribute.
    *
    * @param    max
    *           The new value for the MAX-WIDTH-CHARS attribute.
    */
   public void setMaxWidthChars(double max)
   {
      wcfg.setMaxWidthChars(max);
      LogicalTerminal.pushWindow(wcfg);
   }

   /**
    * Blocks attempts to get the PARENT attribute as it is undefined.
    * 
    * @return  nothing 
    */
   public Object getParent()
   {
      throw new RuntimeException("PARENT attribute is undefined");
   }

   /**
    * Blocks attempts to set the PARENT attribute as it is undefined.
    * 
    * @param   p
    *          Parent widget. 
    */
   public void setParent(Object p)
   {
      throw new RuntimeException("PARENT attribute is undefined");
   }

   /**
    * If <code>visible</code> is <code>false</code> then it has the same
    * effect as the {@link LogicalTerminal#hideAll(boolean)}.
    *
    * @param visible
    *        if <code>false</code> then it hides all widgets and messages.
    *        Otherwise call is ignored.
    */
   public void setVisible(boolean visible)
   {
      if (!visible)
      {
         LogicalTerminal.hideWindow();
      }
   }

   /**
    * Set maximal height.
    * 
    * @param   decimal
    *          New option value.
    */
   public void setMaxHeight(NumberType decimal)
   {
      throw new UnsupportedOperationException("Not implemented");
   }

   /**
    * Set maximal width.
    * 
    * @param   decimal
    *          New option value.
    */
   public void setMaxWidth(NumberType decimal)
   {
      throw new UnsupportedOperationException("Not implemented");
   }

   /**
    * Set virtual height.
    * 
    * @param   decimal
    *          New option value.
    */
   public void setVirtHeight(NumberType decimal)
   {
      throw new UnsupportedOperationException("Not implemented");
   }

   /**
    * Set virtual width.
    * 
    * @param   decimal
    *          New option value.
    */
   public void setVirtWidth(NumberType decimal)
   {
      throw new UnsupportedOperationException("Not implemented");
   }

   /**
    * Enable/disable window resizing.
    * 
    * @param   logical
    *          New option value.
    */
   public void setResize(logical logical)
   {
      throw new UnsupportedOperationException("Not implemented");
   }

   /**
    * Enable/disable displaying of scrollbars.
    * 
    * @param   logical
    *          New option value.
    */
   public void setScrollBars(logical logical)
   {
      throw new UnsupportedOperationException("Not implemented");
   }

   /**
    * Enable/disable displaying of status area.
    * 
    * @param   logical
    *          New option value.
    */
   public void setStatusArea(logical logical)
   {
      throw new UnsupportedOperationException("Not implemented");
   }

   /**
    * Set (default) background color. 
    */
   public void setBgColor()
   {
      throw new UnsupportedOperationException("Not implemented");
   }

   /**
    * Set background color.
    * 
    * @param   value
    *          New color value. 
    */
   public void setBgColor(NumberType value)
   {
      throw new UnsupportedOperationException("Not implemented");
   }
   
   /**
    * Set (default) foreground color.
    */
   public void setFgColor()
   {
      throw new UnsupportedOperationException("Not implemented");
   }

   /**
    * Set foreground color.
    * 
    * @param   value
    *          New color value. 
    */
   public void setFgColor(NumberType value)
   {
      throw new UnsupportedOperationException("Not implemented");
   }
   
   /**
    * Enable/disable KEEP-Z-ORDER option.
    * 
    * @param   logical
    *          New flag value.
    */
   public void setKeepZOrder(logical logical)
   {
      throw new UnsupportedOperationException("Not implemented");
   }

   /**
    * Enable/disable 3D look.
    * 
    * @param   logical
    *          New flag value.
    */
   public void setThreeD(logical logical)
   {
      throw new UnsupportedOperationException("Not implemented");
   }

   /**
    * Enable/disable displaying of message area.
    * 
    * @param   logical
    *          New flag value.
    */
   public void setMessageArea(logical logical)
   {
      throw new UnsupportedOperationException("Not implemented");
   }

   /**
    * Get window maximal height.
    * 
    * @return  maximal height.
    */
   public decimal getMaxHeight()
   {
      throw new UnsupportedOperationException("Not implemented");
   }

   /**
    * Get window maximal width.
    * 
    * @return  maximal width.
    */
   public decimal getMaxWidth()
   {
      throw new UnsupportedOperationException("Not implemented");
   }

   /**
    * Get window virtual height.
    * 
    * @return  virtual height.
    */
   public decimal getVirtHeight()
   {
      throw new UnsupportedOperationException("Not implemented");
   }

   /**
    * Get window virtual width.
    * 
    * @return  virtual width.
    */
   public decimal getVirtWidth()
   {
      throw new UnsupportedOperationException("Not implemented");
   }

   /**
    * Check if window size can be changed.
    * 
    * @return  <code>true</code> if window size can be changed.
    */
   public logical isResize()
   {
      throw new UnsupportedOperationException("Not implemented");
   }

   /**
    * Check if window has scrollbars.
    * 
    * @return  <code>true</code> if window has scrollbars.
    */
   public logical isScrollBars()
   {
      throw new UnsupportedOperationException("Not implemented");
   }

   /**
    * Check if window has message area.
    * 
    * @return  <code>true</code> if window has message area.
    */
   public logical isStatusArea()
   {
      throw new UnsupportedOperationException("Not implemented");
   }

   /**
    * Get background color.
    * 
    * @return  background color. 
    */
   public integer getBgColor()
   {
      throw new UnsupportedOperationException("Not implemented");
   }

   /**
    * Get foreground color
    *  
    * @return  foreground color. 
    */
   public integer getFgColor()
   {
      throw new UnsupportedOperationException("Not implemented");
   }

   /**
    * Check if window has KEEP-Z-ORDER option set.
    * 
    * @return <code>true</code> if window has KEEP-Z-ORDER option set.
    */
   public logical isKeepZOrder()
   {
      throw new UnsupportedOperationException("Not implemented");
   }

   /**
    * Check if window has THREE-D option set.
    * 
    * @return <code>true</code> if window has THREE-D option set.
    */
   public logical isThreeD()
   {
      throw new UnsupportedOperationException("Not implemented");
   }

   /**
    * Check if window has message area.
    * 
    * @return <code>true</code> if window has message area.
    */
   public logical isMessageArea()
   {
      throw new UnsupportedOperationException("Not implemented");
   }

   /**
    * Get icon file name previously loaded by {@link #loadIcon(character)}.
    * 
    * @return  icon file name.
    */
   public character getIcon()
   {
      throw new UnsupportedOperationException("Not implemented");
   }

   /**
    * Load icon for window.
    * 
    * @param   searchPath
    *          Icon file name.
    *          
    * @return  <code>true</code> if operation was successful.
    */
   public logical loadIcon(character searchPath)
   {
      throw new UnsupportedOperationException("Not implemented");
   }

   /**
    * Return reference to {@link WindowWidget} as reference to 
    * {@link GenericWidget}.
    * 
    * @return  this.
    */
   public GenericWidget asWidget()
   {
      return this;
   }
}
